"""A workflow that modifies the Kentik license of a router."""

import logging
from typing import Any

from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State, UUIDstr
from orchestrator.utils.errors import ProcessFailureError
from orchestrator.workflow import StepList, begin, done, step, workflow
from orchestrator.workflows.steps import store_process_subscription
from orchestrator.workflows.utils import wrap_modify_initial_input_form
from pydantic import model_validator
from pydantic_forms.validators import Choice

from gso.products.product_blocks.router import RouterRole
from gso.products.product_types.router import Router
from gso.services.kentik_client import KentikClient

logger = logging.getLogger()


def _initial_input_form(subscription_id: UUIDstr) -> FormGenerator:
    router = Router.from_subscription(subscription_id)
    active_kentik_plans = {str(plan["id"]): plan["name"] for plan in KentikClient().get_plans() if plan["active"]}
    available_kentik_plans = Choice(
        "Select a Kentik license",
        zip(active_kentik_plans.keys(), active_kentik_plans.items(), strict=True),  # type: ignore[arg-type]
    )

    class ModifyKentikLicenseForm(FormPage):
        new_plan_id: available_kentik_plans  # type: ignore[valid-type]

        @model_validator(mode="before")
        def router_must_be_nokia_p(cls, data: Any) -> Any:
            if router.router.router_role != RouterRole.PE:
                msg = "Can only update Kentik license for PE routers."
                raise ValueError(msg)

            return data

    user_input = yield ModifyKentikLicenseForm

    return user_input.model_dump() | {"subscription": router}


@step("Update device license in Kentik")
def update_kentik_license(subscription: Router, new_plan_id: int) -> State:
    """Update a Kentik device with a new license attached to it."""
    kentik_client = KentikClient()
    kentik_device = kentik_client.get_device_by_name(subscription.router.router_fqdn)
    if "id" not in kentik_device:
        msg = "Failed to find Kentik device by name"
        raise ProcessFailureError(msg, details=subscription.router.router_fqdn)

    updated_kentik_device = kentik_client.update_device(kentik_device["id"], {"device": {"plan_id": new_plan_id}})

    return {"kentik_device": updated_kentik_device}


@workflow(
    "Modify Kentik license",
    initial_input_form=wrap_modify_initial_input_form(_initial_input_form),
    target=Target.MODIFY,
)
def modify_router_kentik_license() -> StepList:
    """Apply a selected Kentik license on an existing PE router."""
    return begin >> store_process_subscription(Target.MODIFY) >> update_kentik_license >> done
